package com.ruoyi.system.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Objects;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.system.domain.PostCollectionsSystem;
import com.ruoyi.system.domain.vo.PostCollectionsVo;
import com.ruoyi.system.mapper.PostCollectionsSystemMapper;
import com.ruoyi.system.service.IIssuerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.PostCollectionsMapper;
import com.ruoyi.system.domain.PostCollections;
import com.ruoyi.system.service.IPostCollectionsService;

import static com.ruoyi.common.utils.SecurityUtils.getUsername;

/**
 * 藏品Service业务层处理
 *
 * @author ruoyi
 * @date 2023-02-15
 */
@Service
public class PostCollectionsServiceImpl implements IPostCollectionsService
{
    @Autowired
    private PostCollectionsMapper postCollectionsMapper;
    @Autowired
    private IIssuerService iIssuerService;
    @Autowired
    private PostCollectionsSystemMapper postCollectionsSystemMapper;
    /**
     * 查询藏品详细信息
     *
     * @param id 藏品主键
     * @return 藏品
     */
    @Override
    public PostCollectionsVo selectPostCollectionsById(Long id)
    {
        return postCollectionsMapper.selectPostCollectionsById(id);
    }

    /**
     * 查询藏品列表
     *
     * @param systemId 藏品
     * @return 藏品
     */
    @Override
    public List<PostCollections> selectPostCollectionsList(Long systemId)
    {
        PostCollections postCollections = new PostCollections();
        String postCollectionsSystemStatus = postCollectionsSystemMapper.selectPostCollectionsSystemByStatus(systemId);
        //判断藏品套系是否上架，如果没上架，令藏品status全部置为未上架
        if (Objects.equals(postCollectionsSystemStatus, "0")){
            //批量修改status
            postCollectionsMapper.updatePostCollectionsStatus(systemId);
        }
        //systemId是套系里的主键id
        postCollections.setSystemId(systemId);
        return postCollectionsMapper.selectPostCollectionsList(systemId);
    }

    /**
     * 新增藏品
     *
     * @param postCollectionsVo 藏品
     * @return 结果
     */
    @Override
    public int insertPostCollections(PostCollectionsVo postCollectionsVo)
    {
        Date nowDate = DateUtils.getNowDate();
        Date startTime =  postCollectionsVo.getStartTime();
        Date endTime =  postCollectionsVo.getEndTime();
        //获取此藏品套系status
        Long systemId = postCollectionsVo.getSystemId();
        String postCollectionsSystemStatus = postCollectionsSystemMapper.selectPostCollectionsSystemByStatus(systemId);
        //判断藏品套系是否上架，如果没上架，令藏品也未上架
        if (Objects.equals(postCollectionsSystemStatus, "0")){
            postCollectionsVo.setStatus("4");//未上架
        }
        //如果套系已上架，判断以下时间线
        else {
            //判空
            if (null!=startTime && null!=endTime){

                /**
                 * 判断时间 （0预售 /1已售尽 /2正在售卖 /3已过期）
                 * 根据当前时间，对比售卖时间，
                 * 当前时间小于售卖时间，该套系藏品为预售，
                 * 当前时间大于售卖时间为已过期，
                 * 当前时间在售卖时间之间并且数量大于0为正在售卖
                 * 其余情况已售尽
                 */
                if (nowDate.before(startTime)){
                    postCollectionsVo.setStatus("0");//预售
                }
                else if (nowDate.after(endTime)){
                    postCollectionsVo.setStatus("3");//已过期
                }
                else if (startTime.before(nowDate) &&
                        endTime.after(nowDate) &&
                        postCollectionsVo.getCollectionsNumber()>0){
                    postCollectionsVo.setStatus("2");//正在售卖
                }
                else if(postCollectionsVo.getCollectionsNumber()==0){
                    postCollectionsVo.setStatus("1");//售尽
                }
            }
        }
        postCollectionsVo.setCreateBy(getUsername());
        postCollectionsVo.setUpdateBy(getUsername());
        postCollectionsVo.setUpdateTime(DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,DateUtils.getTime()));
        postCollectionsVo.setCreateTime(DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,DateUtils.getTime()));
        //通过发行方的名字查询id存进数据库
        postCollectionsVo.setSystemId(postCollectionsVo.getSystemId());
        postCollectionsVo.setIssuerId(iIssuerService.selectIssuerByIdForIssuerName(postCollectionsVo.getIssuerName()));

        return postCollectionsMapper.insertPostCollections(postCollectionsVo);
    }

    /**
     * 修改藏品
     *
     * @param postCollections 藏品
     * @return 结果
     */
    @Override
    public int updatePostCollections(PostCollections postCollections)
    {
        postCollections.setUpdateBy(getUsername());
        postCollections.setUpdateTime(DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS,DateUtils.getTime()));
        return postCollectionsMapper.updatePostCollections(postCollections);
    }

    /**
     * 批量删除藏品
     *
     * @param ids 需要删除的藏品主键
     * @return 结果
     */
    @Override
    public int deletePostCollectionsByIds(Long[] ids)
    {
        return postCollectionsMapper.deletePostCollectionsByIds(ids);
    }

    /**
     * 删除藏品信息
     *
     * @param id 藏品主键
     * @return 结果
     */
    @Override
    public int deletePostCollectionsById(Long id)
    {
        return postCollectionsMapper.deletePostCollectionsById(id);
    }
    /**
     * 根据藏品名称查询藏品数量
     *
     * @param postCollectionsVo 藏品
     * @return 结果
     */
    @Override
    public int selectPostCollectionsByName(PostCollectionsVo postCollectionsVo) {
        return postCollectionsMapper.selectPostCollectionsByName(postCollectionsVo);
    }
}
